GLAUNCH_VERSION	= "0.9.8"
GLAUNCH_SHORT_VERSION = 098

# Uncomment one target
#TARGET		= DJGPP
TARGET		= MINGW
#TARGET		= UNIX

# To disable feature, set to zero (0)
USE_TTF		= 1
USE_PNG		= 1
USE_MP3		= 1
USE_JPG		= 1
USE_CD		= 0

DEBUG_MODE	= 1

# ----------------------------------------------------------------------
#
# Nothing to configure below here
#
# ----------------------------------------------------------------------

default: all


GLAUNCH_SOURCE_ZIP	= gl$(GLAUNCH_SHORT_VERSION)s.zip

# ----------------------------------------------------------------------
#
# DJGPP Section
#
# ----------------------------------------------------------------------

ifeq ($(TARGET), DJGPP)
CC		= gcc
CXX		= gxx
LD		= gxx
LD16		= val
USE_UPX		= 1
TARGET_CFLAGS	= -DTARGET_DJGPP

ifeq ($(DEBUG_MODE), 1)
ALLEGRO_LDFLAGS	+= -lalld
else
ALLEGRO_LDFLAGS	+= -lalleg
endif

GLAUNCH_BIN	= glaunch.exe
_GL32_BIN	= _gl32.exe
MAMESCAN_BIN	= mamescan.exe
GLAUNCH_ZIP	= gl$(GLAUNCH_SHORT_VERSION).zip

GLAUNCH_OBJS	= glaunch.obj

$(GLAUNCH_BIN): $(GLAUNCH_OBJS)
	$(LD16) glaunch.obj';;;'
endif

# ----------------------------------------------------------------------
#
# Mingw Section
#
# ----------------------------------------------------------------------

ifeq ($(TARGET), MINGW)
CC		= gcc
CXX		= g++
LD		= g++
USE_UPX		= 1
TARGET_CFLAGS	= -DTARGET_MINGW -DALLEGRO_STATICLINK

WIN32_LIBS	= -lkernel32 -luser32 -lgdi32 -lcomdlg32 -lole32 -ldinput \
		-lddraw -ldxguid -lwinmm -ldsound

ifeq ($(DEBUG_MODE), 1)
ALLEGRO_LDFLAGS	+= -lalld_s $(WIN32_LIBS)
else
ALLEGRO_LDFLAGS	+= -lalleg_s $(WIN32_LIBS)
endif

GLAUNCH_BIN	= glaunch.exe
_GL32_BIN	= _gl32.exe
MAMESCAN_BIN	= mamescan.exe
GLAUNCH_ZIP	= gl$(GLAUNCH_SHORT_VERSION)_win.zip

$(GLAUNCH_BIN): startup/windows/startup_glaunch.exe

startup/windows/startup_glaunch.exe: _startup_exe
	cp -u startup/windows/startup_glaunch.exe glaunch.exe

_startup_exe:
	$(MAKE) -C startup/windows

endif

# ----------------------------------------------------------------------
#
# Unix Section
#
# ----------------------------------------------------------------------

ifeq ($(TARGET), UNIX)
CC		= gcc
CXX		= g++
LD		= g++
USE_UPX		= 0
TARGET_CFLAGS	= -DTARGET_UNIX

ifeq ($(DEBUG_MODE), 1)
ALLEGRO_LDFLAGS	+= `allegro-config --libs`
else
ALLEGRO_LDFLAGS	+= `allegro-config --libs`
endif

GLAUNCH_BIN	= glaunch
GLAUNCH_SRC	= startup/unix/glaunch.cc
GLAUNCH_CFLAGS	= -DGLAUNCH_VERSION='$(GLAUNCH_VERSION)'  -Wall \
	-Wstrict-prototypes -Iinclude
_GL32_BIN	= _gl32
MAMESCAN_BIN	= mamescan.exe

GLAUNCH_SCRIPT	= startup/unix/glaunch.pl

$(GLAUNCH_BIN): $(GLAUNCH_SRC)
	$(CXX) -o $(GLAUNCH_BIN) $(GLAUNCH_CFLAGS) $(GLAUNCH_SRC)
endif

# ----------------------------------------------------------------------
#
# Common
#
# ----------------------------------------------------------------------

CFLAGS 		= -Wall \
		-Wstrict-prototypes \
		-DGLAUNCH_VERSION='$(GLAUNCH_VERSION)' $(INCLUDE_FLAGS) \
		-DUSE_MP3=$(USE_MP3) -DUSE_TTF=$(USE_TTF) \
		-DUSE_PNG=$(USE_PNG) -DUSE_JPG=$(USE_JPG) \
		-DUSE_CD=$(USE_CD) $(TARGET_CFLAGS)

DEBUG_CFLAGS	= -g
OPTIMIZE_CFLAGS	= -Wstrict-prototypes \
		-O2 \
		-fomit-frame-pointer \
		-ffast-math \
		-fexpensive-optimizations \
		-fpeephole \
		-funroll-loops


CXXFLAGS 	= $(CFLAGS)

LDFLAGS 	= $(TARGET_LDFLAGS)
DEBUG_LDFLAGS	= -g

ASM 		= nasm
ASMFLAGS 	= -f obj

UPX 		= upx --best
RM		= rm -f
ZIP		= zip

HTML2TXT	= lynx -dump -nolist


MAKE_DEPEND_D	= $(RM) depend.d ; \
		gcc -MM $(INCLUDE_FLAGS) -DTARGET=$(TARGET) $(SRCS) >> depend.d

ifeq ($(DEBUG_MODE), 1)
CFLAGS		+= $(DEBUG_CFLAGS)
LDFLAGS		+= $(DEBUG_LDFLAGS)
else
CFLAGS		+= $(OPTIMIZE_CFLAGS)
endif

default: all

clean: common_clean

common_clean:
	$(RM) depend.d *~ *.o *.obj *.png crash*.log

%.o: %.c
	$(CC)  $(CFLAGS) -c $< -o $@

%.o: %.cc
	$(CXX)  $(CXXFLAGS) -c $< -o $@

%.o: %.s
	$(CC) -x assembler-with-cpp $(CFLAGS) -o $@ -c $<

%.obj: %.asm
	$(ASM) -o $@ $(ASMFLAGS) $<

%.txt: %.htm
	$(HTML2TXT) $< > $@
