#include <map>
#include "gl_key.h"
#include "allegro.h"

typedef map<int, int> GlKeyMap;

static GlKeyMap * glToAllegroMap = 0;

typedef struct
{
    int glScancode;
    int allegroScancode;
} GlKeyInfo;

static GlKeyInfo glToAllegroTable[] =
{
    {GL_KEY_A,              KEY_A},
    {GL_KEY_B,              KEY_B},
    {GL_KEY_C,              KEY_C},           
    {GL_KEY_D,              KEY_D},
    {GL_KEY_E,              KEY_E},
    {GL_KEY_F,              KEY_F},
    {GL_KEY_G,              KEY_G},
    {GL_KEY_H,              KEY_H},
    {GL_KEY_I,              KEY_I},
    {GL_KEY_J,              KEY_J},
    {GL_KEY_K,              KEY_K},
    {GL_KEY_L,              KEY_L},
    {GL_KEY_M,              KEY_M},
    {GL_KEY_N,              KEY_N},
    {GL_KEY_O,              KEY_O},
    {GL_KEY_P,              KEY_P},
    {GL_KEY_Q,              KEY_Q},
    {GL_KEY_R,              KEY_R},
    {GL_KEY_S,              KEY_S},
    {GL_KEY_T,              KEY_T},
    {GL_KEY_U,              KEY_U},
    {GL_KEY_V,              KEY_V},
    {GL_KEY_W,              KEY_W},
    {GL_KEY_X,              KEY_X},
    {GL_KEY_Y,              KEY_Y},
    {GL_KEY_Z,              KEY_Z},
    {GL_KEY_0,              KEY_0},
    {GL_KEY_1,              KEY_1},
    {GL_KEY_2,              KEY_2},
    {GL_KEY_3,              KEY_3},
    {GL_KEY_4,              KEY_4},
    {GL_KEY_5,              KEY_5},
    {GL_KEY_6,              KEY_6},
    {GL_KEY_7,              KEY_7},
    {GL_KEY_8,              KEY_8},
    {GL_KEY_9,              KEY_9},
    {GL_KEY_0_PAD,          KEY_0_PAD},
    {GL_KEY_1_PAD,          KEY_1_PAD},
    {GL_KEY_2_PAD,          KEY_2_PAD},
    {GL_KEY_3_PAD,          KEY_3_PAD},
    {GL_KEY_4_PAD,          KEY_4_PAD},
    {GL_KEY_5_PAD,          KEY_5_PAD},
    {GL_KEY_6_PAD,          KEY_6_PAD},
    {GL_KEY_7_PAD,          KEY_7_PAD},
    {GL_KEY_8_PAD,          KEY_8_PAD},
    {GL_KEY_9_PAD,          KEY_9_PAD},
    {GL_KEY_F1,             KEY_F1},
    {GL_KEY_F2,             KEY_F2},
    {GL_KEY_F3,             KEY_F3},
    {GL_KEY_F4,             KEY_F4},
    {GL_KEY_F5,             KEY_F5},
    {GL_KEY_F6,             KEY_F6},
    {GL_KEY_F7,             KEY_F7},
    {GL_KEY_F8,             KEY_F8},
    {GL_KEY_F9,             KEY_F9},
    {GL_KEY_F10,            KEY_F10},
    {GL_KEY_F11,            KEY_F11},
    {GL_KEY_F12,            KEY_F12},
    {GL_KEY_ESC,            KEY_ESC},
    {GL_KEY_TILDE,          KEY_TILDE},
    {GL_KEY_MINUS,          KEY_MINUS},
    {GL_KEY_EQUALS,         KEY_EQUALS},
    {GL_KEY_BACKSPACE,      KEY_BACKSPACE},
    {GL_KEY_TAB,            KEY_TAB},
    {GL_KEY_OPENBRACE,      KEY_OPENBRACE},
    {GL_KEY_CLOSEBRACE,     KEY_CLOSEBRACE},
    {GL_KEY_ENTER,          KEY_ENTER},
    {GL_KEY_COLON,          KEY_COLON},
    {GL_KEY_QUOTE,          KEY_QUOTE},
    {GL_KEY_BACKSLASH,      KEY_BACKSLASH},
    {GL_KEY_BACKSLASH2,     KEY_BACKSLASH2},
    {GL_KEY_COMMA,          KEY_COMMA},
    {GL_KEY_STOP,           KEY_STOP},
    {GL_KEY_SLASH,          KEY_SLASH},
    {GL_KEY_SPACE,          KEY_SPACE},
    {GL_KEY_INSERT,         KEY_INSERT},
    {GL_KEY_DEL,            KEY_DEL},
    {GL_KEY_HOME,           KEY_HOME},
    {GL_KEY_END,            KEY_END},
    {GL_KEY_PGUP,           KEY_PGUP},
    {GL_KEY_PGDN,           KEY_PGDN},
    {GL_KEY_LEFT,           KEY_LEFT},
    {GL_KEY_RIGHT,          KEY_RIGHT},
    {GL_KEY_UP,             KEY_UP},
    {GL_KEY_DOWN,           KEY_DOWN},
    {GL_KEY_SLASH_PAD,      KEY_SLASH_PAD},
    {GL_KEY_ASTERISK,       KEY_ASTERISK},
    {GL_KEY_MINUS_PAD,      KEY_MINUS_PAD},
    {GL_KEY_PLUS_PAD,       KEY_PLUS_PAD},
    {GL_KEY_DEL_PAD,        KEY_DEL_PAD},
    {GL_KEY_ENTER_PAD,      KEY_ENTER_PAD},
    {GL_KEY_PRTSCR,         KEY_PRTSCR},
    {GL_KEY_PAUSE,          KEY_PAUSE},
    {GL_KEY_ABNT_C1,        KEY_ABNT_C1},
    {GL_KEY_YEN,            KEY_YEN},
 
    {GL_KEY_LSHIFT,         KEY_LSHIFT},
    {GL_KEY_RSHIFT,         KEY_RSHIFT},
    {GL_KEY_LCONTROL,       KEY_LCONTROL},
    {GL_KEY_RCONTROL,       KEY_RCONTROL},
    {GL_KEY_ALT,            KEY_ALT},
    {GL_KEY_ALTGR,          KEY_ALTGR},
    {GL_KEY_LWIN,           KEY_LWIN},
    {GL_KEY_RWIN,           KEY_RWIN},
    {GL_KEY_MENU,           KEY_MENU},
    {GL_KEY_SCRLOCK,        KEY_SCRLOCK},
    {GL_KEY_NUMLOCK,        KEY_NUMLOCK},
    {GL_KEY_CAPSLOCK,       KEY_CAPSLOCK},
                    
    {GL_KEY_KANA,           KEY_KANA},
    {GL_KEY_CONVERT,        KEY_CONVERT},
    {GL_KEY_NOCONVERT,      KEY_NOCONVERT},
    {GL_KEY_AT,             KEY_AT},
    {GL_KEY_CIRCUMFLEX,     KEY_CIRCUMFLEX},
    {GL_KEY_COLON2,         KEY_COLON2},
    {GL_KEY_KANJI,          KEY_KANJI},
    {0,                     0}
};

static void init_key_map();

int gl_key_to_allegro_key(int gl_scancode)
{
    GlKeyMap::iterator  keyInfo;
    int allegroScancode;

    if (glToAllegroMap == 0)
    {
        init_key_map();
    }

    keyInfo = glToAllegroMap->find(gl_scancode);
    if (keyInfo == glToAllegroMap->end())
    {
        allegroScancode = KEY_SPACE;
    }
    else
    {
        allegroScancode = keyInfo->second;
    }
    
    return allegroScancode;
}

void init_key_map()
{
    glToAllegroMap = new GlKeyMap();
    GlKeyInfo * keyInfo;

    keyInfo = &glToAllegroTable[0];

    while (keyInfo->glScancode != 0)
    {
        (*glToAllegroMap)[keyInfo->glScancode] = keyInfo->allegroScancode;
        keyInfo++;
    }
}
