#include <iostream>
#include <strstream>
#include <fstream>
#include <string>
#include <stdio.h>
#include <unistd.h>

#include "allegro.h"
#if (USE_TTF == 1)
#include "allegttf.h"
#endif
#include "glaunch.h"
#include "gl_gui.h"
#include "fileutil.h"
#include "sshot.h"
#include "menu.h"
#include "mp3bg.h"
#include "cdbg.h"
#include "logfile.h"
#include "view.h"
#include "control.h"

using namespace GameLauncher;
using namespace std;
using LogFile::gl_log;


extern "C" void _antialiased_render (struct BITMAP *bmp, void* glyphs, int ch, int x, int y, int colour);

#define _S(_A_, _B_) \
char * Launcher_Controller:: _A_ = _B_

_S( DATA_FILE,                  "glaunch.dat");
#if defined(TARGET_MINGW) || defined(TARGET_DJGPP)
_S( BATCH_FILE,                 "runit.bat");
#elif defined(TARGET_UNIX)
_S( BATCH_FILE,                 "runit.sh");
#endif
_S( BATCH_LOG_FILE,             "logs/runit.log");
_S( CONFIG_DIR,                 "config");
_S( CONFIG_FILE,                "glaunch.cfg");
_S( STATE_FILE,                 "glaunch.sta");
_S( FONT_PCX_CACHE,             "_font.pcx");

_S( MAIN_SECTION,               "Main");
_S( MAIN_BACKGROUND_MUSIC,      "background_music");
_S( MAIN_USE_SOFTWARE_MIXER,    "use_software_mixer");
_S( MAIN_ANIMATION_METHOD,      "animation_method");
_S( MAIN_USE_VSYNC,             "use_vsync");
_S( MAIN_USE_VRAM,              "use_vram");
_S( MAIN_USE_FULLSCREEN,        "use_fullscreen");
_S( MAIN_MENU_ORIENTATION,      "menu_orientation");
_S( MAIN_FONT_FILE,             "font_file");
_S( MAIN_FONT_SIZE,             "font_size");
_S( MAIN_ANTIALIAS_FONT,        "antialias_font");
_S( MAIN_RESOLUTION,            "resolution");
_S( MAIN_DEBUG,                 "debug");
_S( MAIN_SCREEN_SAVER_TIME,     "screen_saver_time");
_S( MAIN_SCREEN_SAVER_TEXT,     "screen_saver_text");
_S( MAIN_SKIP_SONG_KEY,         "skip_song_key");
_S( MAIN_START_SONG_OVER_KEY,   "start_song_over_key");

_S( STATE_SECTION,              "Main");
_S( STATE_LAST_FONT_FILE,       "last_font_file");
_S( STATE_LAST_FONT_FILE_SIZE,  "last_font_file_size");
_S( STATE_LAST_FONT_SIZE,       "last_font_size");
_S( STATE_LAST_ANTIALIAS_FONT,  "last_antialias_font");

_S( DEFAULT_FONT_FILE,          "slicker.ttf");

#undef _S

Launcher_Controller::Launcher_Controller(bool is_first_time) throw (InitError)
{
    DATAFILE *  data_file;
    char        file_name[MAX_FILE_NAME+1];

    if (is_first_time)
    {
        fade_out(5);
    }

    // Read in configs
    Config_Section *    section;
    vector<string> *    resolution;
    int         background_music;
    bool        use_software_mixer;
    int         animation_method;
    int         menu_orientation;
    bool        use_vsync;
    bool        use_vram;
    int         skip_key_scancode;
    int         start_over_key_scancode;
    bool        use_fullscreen;
    int         screen_saver_time;
    string      screen_saver_text;

    p_config_file       = new Config_File(CONFIG_FILE);
    p_state_file        = new Config_File(STATE_FILE);

    section             = p_config_file->get_section(MAIN_SECTION);
    p_is_debug_on       = section->get_bool(MAIN_DEBUG, false);
    background_music    = section->get_int(MAIN_BACKGROUND_MUSIC,
                                           MAIN_BACKGROUND_MUSIC_NONE);
    use_software_mixer  = section->get_bool(MAIN_USE_SOFTWARE_MIXER, false);
    animation_method    = section->get_int(MAIN_ANIMATION_METHOD,
                                           MAIN_ANIMATION_PAGE_FLIPPING);
    use_vsync           = section->get_bool(MAIN_USE_VSYNC, true);
    use_vram            = section->get_bool(MAIN_USE_VRAM, true);
    use_fullscreen      = section->get_bool(MAIN_USE_FULLSCREEN, true);
    resolution          = section->get_string_list(MAIN_RESOLUTION);
    menu_orientation    = section->get_int(MAIN_MENU_ORIENTATION,
                                           MAIN_HORIZONTAL_MENU);
    p_font_file         = section->get_string(MAIN_FONT_FILE,
                                              DEFAULT_FONT_FILE);
    p_font_size         = section->get_int(MAIN_FONT_SIZE, 20);
    p_antialias_font    = section->get_bool(MAIN_ANTIALIAS_FONT, true);

    screen_saver_time   = section->get_int(MAIN_SCREEN_SAVER_TIME, 5);
    screen_saver_text   = section->get_string(MAIN_SCREEN_SAVER_TEXT, "");

    skip_key_scancode   = section->get_int(MAIN_SKIP_SONG_KEY, 0);
    start_over_key_scancode = section->get_int(MAIN_START_SONG_OVER_KEY, 0);
    delete section;


    // Create the engine
    Engine::Animation_Method    engine_method;
    int engine_width;
    int engine_height;

    if (animation_method == MAIN_ANIMATION_PAGE_FLIPPING)
        engine_method   = Engine::PAGE_FLIPPING;
    else
        engine_method   = Engine::DOUBLE_BUFFERING;

    if (resolution->size() == 2)
    {
        engine_width    = atoi((*resolution)[0].c_str());
        engine_height   = atoi((*resolution)[1].c_str());
    }
    else if (menu_orientation == MAIN_VERTICAL_MENU)
    {
        engine_width    = DEFAULT_VERTICAL_WIDTH;
        engine_height   = DEFAULT_VERTICAL_HEIGHT;
    }
    else
    {
        engine_width    = DEFAULT_HORIZONTAL_WIDTH;
        engine_height   = DEFAULT_HORIZONTAL_HEIGHT;
    }

    {
        // Dammit!  Why aren't there ostringstreams??  And why don't
        // ostrstreams initialize their damn buffer!!
        char            buf[81];
        memset(buf, 0, 81);
        ostrstream      text(buf, 80);

        text << "Using screen resolution of " << engine_width << "x"
             << engine_height;
        gl_log->info(text.str());
    }

    try
    {
        p_engine = Engine::get_instance();
        // Use 16 BPP Color depth
        p_engine->initialize_sound(use_software_mixer);
        p_engine->set_graphics_mode(engine_width, engine_height, 16,
                                    use_fullscreen);
        p_engine->set_animation_mode(engine_method, use_vram, use_vsync);

        // Load the data file, making sure the case and slashes are correct
        strncpy(file_name, (char *) DATA_FILE, MAX_FILE_NAME);
        (void) fix_filename_case(file_name);
        (void) fix_filename_slashes(file_name);
        data_file = load_datafile(file_name);
        if (!data_file)
        {
            ostrstream error;
            error << "Error loading " << DATA_FILE << endl;
            throw InitError(error.str());
        }
        p_data_file = new DataFile(data_file);

        // Delete the batch file.  Doesn't matter if this fails....
        strncpy(file_name, (char *) BATCH_FILE, MAX_FILE_NAME);
        (void) fix_filename_case(file_name);
        (void) fix_filename_slashes(file_name);

        // Keep the batch file if debug is on.
        if (! p_is_debug_on)
        {
            (void) unlink(file_name);
        }


        // Create the models
        p_emulator_manager  = new Emulator_Manager(p_config_file,
                                                   p_state_file);

        // Create the other controllers
#if (USE_MP3 == 1)
        if (background_music == MAIN_BACKGROUND_MUSIC_MP3)
        {
            p_bg_controller = new MP3_Music_Controller(p_engine);
        }
        else
#endif

#if (USE_CD == 1)
	if (background_music == MAIN_BACKGROUND_MUSIC_CD)
        {
            p_bg_controller = new CD_Music_Controller(p_engine);
        }
        else
#endif
        {
            p_bg_controller = new Null_Music_Controller();
            gl_log->info("No background music");
        }

        p_bg_controller->load_config(p_config_file);

        load_font();

        if (menu_orientation == MAIN_VERTICAL_MENU)
        {
            p_emu_menu_controller =
                new Emu_Menu_Controller(p_engine, p_loaded_font,
                                        0, 0,
                                        SCREEN_W, SCREEN_H,
                                        VERTICAL, p_emulator_manager);
        }
        else
        {
#if 1
            p_emu_menu_controller =
                new Emu_Menu_Controller(p_engine, p_loaded_font,
                                        SCREEN_W/4, 0,
                                        (SCREEN_W/2 - 1), SCREEN_H,
                                        HORIZONTAL, p_emulator_manager);
#else
            p_emu_menu_controller =
                new Emu_Menu_Controller(p_engine, p_loaded_font,
                                        0, 0,
                                        SCREEN_W/2-1, SCREEN_H,
                                        HORIZONTAL, p_emulator_manager);
#endif
        }

        p_emu_menu_controller->set_listener(this);

        GEContainer *       root_container;
        root_container      = p_engine->get_root_container();

        p_delay     = new Delay_VC(p_engine, this);
        p_delay->delay(1000);
        root_container->add_updatable(p_delay, UPDATE_PRIORITY);

        p_bg_view   = new Background_Music_VC(p_engine);
        p_bg_view->set_skip_key(skip_key_scancode);
        p_bg_view->set_start_over_key(start_over_key_scancode);
        p_bg_view->set_listener(this);
        root_container->add_updatable(p_bg_view, UPDATE_PRIORITY);

        // The screen saver should always update after the other
        // objects.
        p_screen_saver  = new Screen_Saver_VC(p_engine, p_loaded_font);
        p_screen_saver->set_screen_saver_listener(this);
        p_screen_saver->set_start_after(60 * screen_saver_time);
        p_screen_saver->set_text(screen_saver_text);
        root_container->add_updatable(p_screen_saver, UPDATE_PRIORITY + 1);
    }
    catch (InitError ie)
    {
        set_gfx_mode(GFX_TEXT, 80, 25, 0, 0);
        set_palette(default_palette);
        throw;
    }


    p_state     = INIT;
    return;
}

Launcher_Controller::~Launcher_Controller()
{
    gl_log->info("Cleaning up Launcher_Controller");
    delete p_screen_saver;
    delete p_bg_view;
    delete p_data_file;
    delete p_emu_menu_controller;
    delete p_bg_controller;
    delete p_emulator_manager;
    delete p_config_file;
    delete p_delay;

    // Always deleted last.
    delete p_engine;

#if (USE_TTF == 1)
    // Must be called before allegro_exit() as it calls some Allegro
    // functions.
    antialias_exit();
#endif

    allegro_exit();
    gl_log->info("Cleaning up Launcher_Controller: Done");
}

string
Launcher_Controller::get_version_string()
{
    return (string("Game Launcher v") + GLAUNCH_VERSION);
}

int
Launcher_Controller::launch_game()
{
    restore_state();

    // One of the other controllers will set the exit code accordingly
    // in run(), but just in case it forgets. :)
    p_exit_code = EXIT_TERMINATE;
    p_engine->run();

    save_state();

    gl_log->info("Exitting");
    return p_exit_code;
}

void
Launcher_Controller::game_selected(Emulator * emulator, Game * game)
{
    char        cwd[80];
    string      output_log;
    string      directory;
    string      command;
    string      pre_command;
    string      post_command;

    (void) getcwd(cwd, 80);
    fix_filename_slashes(cwd);

    output_log = string(cwd) + "/" + BATCH_LOG_FILE;
    fix_filename_slashes(output_log);

    ofstream    file(BATCH_FILE);
    if (! file)
        return;
    
    directory   = emulator->directory();
    fix_filename_slashes(directory);
    command     = emulator->get_command_to_run(game);
    pre_command = emulator->get_pre_command();
    post_command = emulator->get_post_command();
    
    gl_log->info("Game selected: <" + game->get_full_name() + ">");
    gl_log->info("Emulator: <" + emulator->name() + ">");
    gl_log->info("Directory: <" + directory + ">");
    gl_log->info("Pre-command: <" + pre_command + ">");
    gl_log->info("Post-command: <" + post_command + ">");
    gl_log->info("Command: <" + command + ">");
    
    write_script_header(file);

    if (! p_is_debug_on)
    {
        // Need to get rotate progress, as this can take a while and
        // cause skipping in the background task.
        LogFile::rotate_log_files(BATCH_LOG_FILE, 5, this);
    }
    
    if (pre_command != "")
    {
        write_command(file, pre_command, "");
    }
    file << "cd " << directory << endl;
    write_command(file, command, output_log);
    file << "cd " << cwd << endl;
    if (post_command != "")
    {
        write_command(file, post_command, "");
    }

    if (p_is_debug_on)
    {
        // Give user one last look at the output
        file << "pause\n";
        p_exit_code = EXIT_RUN_DEBUGGED_BATCH_FILE;
    }
    else
    {
        p_exit_code = EXIT_RUN_BATCH_FILE;
    }

    file.close();

    p_state     = DONE;
    p_delay->delay(250);
}

void
Launcher_Controller::write_script_header(ostream & file)
{
#if defined(TARGET_DJGPP) || defined(TARGET_MINGW)
    if (! p_is_debug_on)
    {
        file << "@echo off\n";
    }
    else
    {
        file << "@echo on\n";
    }
#elif defined(TARGET_UNIX)
    file << "#!/bin/sh\n\n";
    if (p_is_debug_on)
    {
        file << "pause () {\n";
        file << "  echo \"Press return to continue...\"\n";
        file << "  read\n";
        file << "}\n\n";
    }
#endif
}

void
Launcher_Controller::write_command(ofstream & file, const string & command,
                                   const string & output_log)
{
#if defined(TARGET_UNIX)
    string stderr_redirect = " 2>&1";
#else
    string stderr_redirect = "";
#endif

    if (p_is_debug_on)
    {
        file << "echo Will run: \"" << command << "\"\n";
        file << "pause\n";
        file << command << endl;
    }
    else
    {
        if (output_log != "")
            file << command << " > " << output_log << stderr_redirect << endl;
        else
            file << command << endl;
    }
}

void
Launcher_Controller::rotate_progress(const Log_Rotate_Event *)
{
    p_engine->poll_background_task();
}

void
Launcher_Controller::quit_selected()
{
    gl_log->info("Quit selected");
    p_exit_code = EXIT_TERMINATE;
    p_state     = DONE;
    p_delay->delay(250);
}


void
Launcher_Controller::skip_to_next_song(const Background_Music_Event * e)
{
    p_bg_controller->skip_to_next_song();
}

void
Launcher_Controller::start_song_over(const Background_Music_Event * e)
{
    p_bg_controller->start_song_over();
}

void
Launcher_Controller::screen_saver_changed_state(const Screen_Saver_Event * e)
{
    GEContainer *       root;

    root = p_engine->get_root_container();
    if (e->is_active())
    {
        p_emu_menu_controller->set_views_hidden(true);
        root->add_drawable(p_screen_saver, DRAW_PRIORITY);
    }
    else
    {
        root->remove_drawable(p_screen_saver);
        p_emu_menu_controller->set_views_hidden(false);
    }
}

void
Launcher_Controller::delay_vc_expired()
{
    if (p_state == INIT)
    {
        p_state = RUNNING;
        p_emu_menu_controller->start();
    }
    else if (p_state == DONE)
    {
        p_engine->stop();
    }
}

void
Launcher_Controller::restore_state()
{
    // Restore model classes before controller classes
    p_emulator_manager->restore_state();

    p_bg_controller->restore_state(p_state_file);
    p_emu_menu_controller->load_state(p_data_file, p_config_file);
}

void
Launcher_Controller::save_state()
{
    p_bg_controller->save_state(p_state_file);
    p_emulator_manager->save_state();

    Config_Section *    section;
    section             = p_state_file->get_section(STATE_SECTION);
    section->set_string(STATE_LAST_FONT_FILE, p_font_file);
    section->set_int(STATE_LAST_FONT_FILE_SIZE, p_font_file_size);
    section->set_int(STATE_LAST_FONT_SIZE, p_font_size);
    section->set_bool(STATE_LAST_ANTIALIAS_FONT, p_antialias_font);
    delete section;
}

void
Launcher_Controller::load_font() throw (InitError)
{
    string      last_font_file;
    int         last_font_file_size;
    int         last_font_size;
    bool        last_antialias_font;

    Config_Section *    section;
    section             = p_state_file->get_section(STATE_SECTION);
    last_font_file      = section->get_string(STATE_LAST_FONT_FILE, "");
    last_font_file_size = section->get_int(STATE_LAST_FONT_FILE_SIZE, 0);
    last_font_size      = section->get_int(STATE_LAST_FONT_SIZE, -1);
    last_antialias_font = section->get_bool(STATE_LAST_ANTIALIAS_FONT, false);
    delete section;

#if (USE_TTF == 0)
    p_loaded_font = p_data_file->list_font();
#else
    // Drop back to default font, if the user-defined font does not exist
    if (! file_exists(p_font_file))
    {
        // Give up if the default font is gone
        if (! file_exists(DEFAULT_FONT_FILE))
        {
            throw InitError("No font found");
        }
            
        p_font_file     = DEFAULT_FONT_FILE;
    }

    p_font_file_size    = file_size(p_font_file);

#if 0
    if ((p_font_file == last_font_file) &&
        (p_font_file_size == last_font_file_size) &&
        (p_font_size == last_font_size) &&
        (p_antialias_font == last_antialias_font) &&
        (file_exists((char *) FONT_PCX_CACHE, 0, 0)))
    {
        gl_log->info("Loading font from cache");
        p_loaded_font = ::load_font(FONT_PCX_CACHE);
        if (p_antialias_font)
            p_loaded_font->renderhook = _antialiased_render;
        gl_log->info("Font loaded");
    }
    else
    {
#endif
        gl_log->info("Loading font: " + p_font_file);
        int smooth_mode;

        if (p_antialias_font)
            smooth_mode = ALLEGTTF_REALSMOOTH;
        else
            smooth_mode = ALLEGTTF_NOSMOOTH;

        p_loaded_font = load_ttf_font(p_font_file.c_str(), p_font_size,
                                      smooth_mode);
        gl_log->info("Font loaded");
#if 0
        save_pcx_font();
    }
#endif
#endif // USE_TTF
}

#if 0
void
Launcher_Controller::save_pcx_font()
{
    BITMAP *    thechar;
    BITMAP *    dest;
    int         w;
    int         h;
    int         bw;
    int         bh;
    int         i;
    int         startchar;
    int         endchar;

    // Setup a nice black-white gradient palette
    for(i=0;i<256;i++)
    {
        desktop_palette[i].r = i/4;
        desktop_palette[i].g = i/4;
        desktop_palette[i].b = i/4;
    }

    // Figure out the size of the destination bitmap
    thechar = (BITMAP*)(p_loaded_font->glyphs[0]);
    w = ((thechar->w*2)+16) & 0xFFF0;
    h = ((thechar->h)+16) & 0xFFF0;

    bw = 1+w*16;
    bh = 1+h*((p_font_size+15)/16) * 12;

    // Make a bit destination bitmap
    dest = create_bitmap_ex(8,bw,bh);
    if(dest==NULL)
    {
        fprintf(stderr,"Could not allocate bitmap");
        exit(0);
    }
    clear_to_color(dest,255);


    if(p_antialias_font)
      antialias_init(desktop_palette);
   
    startchar   = 32;
    endchar     = 128;
    for(i = startchar; i < endchar; i++)
    {
        int x = 1+w*(i&15);
        int y = 1+h*(i/16);

        // Put the character onto the bitmap
        thechar = (BITMAP*)p_loaded_font->glyphs[i-startchar];
        
        if(thechar!=NULL)
            blit(thechar,dest,0,0,x,y,thechar->w,thechar->h);
    }

    // Save the PCX file out
    save_pcx(FONT_PCX_CACHE, dest, desktop_palette);

    destroy_bitmap(dest);
    gl_log->info("Cached font");
}
#endif
