#include <iostream>

#include "control.h"
#include "logfile.h"

using namespace GameLauncher;
using LogFile::gl_log;

static bool quietly_create_directory(const string & directory);

int main(int argc, char * argv[])
{
    bool arguments_are_invalid = true;
    bool is_first_time = false;
    if (argc == 3)
    {
        // Called like "0x0bad 0xcafe" the first time we are called,
        // and "0xdead 0xbeef" after that.

        if ((strcmp(argv[1], "0x0bad") == 0) &&
            (strcmp(argv[2], "0xcafe") == 0))
        {
            arguments_are_invalid = false;
            is_first_time = true;
        }
        else if ((strcmp(argv[1], "0xdead") == 0) &&
            (strcmp(argv[2], "0xbeef") == 0))
        {
            arguments_are_invalid = false;
            is_first_time = false;
        }
    }
       
    allegro_init();

    if (arguments_are_invalid)
    {
        allegro_message("Use glaunch.exe.\n");
        return EXIT_ERROR;
    }

    if (! quietly_create_directory("logs"))
    {
        return EXIT_NO_LOG_DIRECTORY;
    }

    int rc;
    Launcher_Controller * launcher = 0;
    try
    {
        LogFile::open_log_file("logs/glaunch.log");

        gl_log->info(Launcher_Controller::get_version_string());
        gl_log->info(string(allegro_id));

        launcher = new Launcher_Controller(is_first_time);
        rc = launcher->launch_game();
    }
    catch (InitError & e)
    {
        gl_log->error(e.error_string());
        rc = EXIT_ERROR;
    }
    catch (...)
    {
        rc = EXIT_ERROR;
    }

    if (launcher != 0)
    {
        delete launcher;
        launcher = 0;
    }

    return rc;
}

END_OF_MAIN();

static bool quietly_create_directory(const string & directory)
{
    int rc = true;
    if (! directory_exists(directory))
    {
        if (!create_directory(directory))
        {
            rc = false;
        }
        else if (!directory_exists(directory))
        {
            rc = false;
        }
    }

    return rc;
}
