
	                segment code 
        
stack_size:	equ	0x100

..start:	
	global	_main
_main:	
	mov	ax, data		; Load data segment
	mov	ds, ax			;  .
	mov	ax, stack		; Load stack segment
	mov	ss, ax			;  .
	mov	sp, stack_top		; Load stack pointer

	;; Call hide_cursor();
	call	_hide_cursor

	;; Call dprint(loading_msg);
	push	word loading_msg
	call	_dprint
	add	sp, 2

	;; MS-DOS allocates all available memory to a program upon loading.
	;; We must deallocate all unused memory, otherwise the spawned
	;; program will not have any for itself.
	
	;; The first address of our program is ES:0000.
	;; The last address of our program is SS:stack_top.
	;; Calculate the difference to find out how much space is neede
 	mov	ax, ss			; Get SS (which is in paragraphs)
	mov	bx, stack_top		; Divide stack_top by 16 to get
	mov	cx, 4			;  SS:stack_top in paragraphs
	shr	bx, cl			;  .
	add	ax, bx			; AX holds very last para address
	inc	ax			;  .
	mov	bx, es			; BX holds very first para address
 	sub	ax, bx			; Calculate the difference
	
	;; Call free_memory(ax);
	push	ax
	call	_free_memory
	add	sp, 2
	cmp	ax, 0			; Check return code for error
	je	.exec_first_menu	; Jump ahead, if OK

.print_free_memory_error:	
	;; dprint(memory_error_msg);
	mov	bx, ax			; Save error code in BX
	push	word memory_error_msg	; Print memory_error_msg
	call	_dprint			;  .
	add	sp, 2			;  ..
	
	;; print_hex16(error);
	push	bx			; Print error code
	call	_print_hex16		;  .
	add	sp, 2			;  ..
	jmp	.terminate
	
	
.exec_first_menu:	
	;; Call hide_cursor();
	call	_hide_cursor
	
	;; Call exec(first_exec_menu, first_menu_prog);
	push	word first_menu_prog
	push	word exec_first_menu
	call	_exec
	jmp	.exec_common
	
.exec_menu:	
	;; Call hide_cursor();
	call	_hide_cursor
	
	;; Call exec(exec_menu, menu_prog);
	push	word menu_prog
	push	word exec_menu
	call	_exec
	
.exec_common:	
	add	sp, 4
	cmp	ax, 0			; See if exec() failed
	jne	.print_exec_error	;  .
	call	_get_rc			; Check exit code
	cmp	ax, 0			; if(EXIT_RUN_BATCH_FILE)
	je	.exec_batch		;  exec(runit.bat)
	cmp	ax, 1			; elif(EXIT_RUN_DEBUGGED_BATCH_FILE)
	je	.exec_batch		;  exec(runit.bat)
	cmp	ax, 4			; elif(EXIT_NO_LOG_DIRECTORY)
	je	.print_no_log_error	;  print log error
	cmp	ax, 2			; elif(EXIT_TERMINATE)
	je	.terminate		;  .
	jmp	.print_exec_error	; else fall through to failure msg
	
.print_exec_error:	
	;; dprint(exec_msg);
	mov	bx, ax			; Save error code in BX
	push	word exec_msg		; Print exec_msg
	call	_dprint			;  .
	add	sp, 2			;  ..
	
	;; print_hex16(error);
;	push	bx			; Print error code
;	call	_print_hex16		;  .
;	add	sp, 2			;  ..
	
	jmp	.terminate		; Don't continue running
	
.print_no_log_error:
	;; dprint(no_log_msg);
	push	word no_log_msg
	call	_dprint
	add	sp, 2
	jmp	.terminate

.exec_batch:	
	;; Call hide_cursor();
	call	_hide_cursor
	
	;; Call exec(exec_batch, batch_prog);
	push	word batch_prog
	push	word exec_batch
	call	_exec
	add	sp, 4
	cmp	ax, 0
	jne	.terminate

	jmp	.exec_menu
	
.terminate:	
	;; Call show_cursor();
	call	_show_cursor
	
	mov	ax, 0x4C00		; Terminate the program
	int	0x21			;  .
	
;;; 
;;; void dprint(DOS_STRING * string)
;;; 
	global	_dprint
_dprint:
	enter	0, 0			; Setup registers
	push	dx			;  .
	
	mov	dx, [bp+4]		; Move string pointer into DX
	mov	ah, 0x09		; Call DOS print string
	int	0x21			;  .
	
	pop	dx			; Restore registers
	leave				;  .
	ret

;;;
;;; uInt16 free_memory(uInt16 paragraphs)
;;; 
	global	_free_memory
_free_memory:
	enter	0, 0
	push	bx
	
	mov	bx, [bp+4]		; Move `paragraphs' into BX
 	mov	ah, 0x4A		; Delallocte memory
 	int	0x21			;  ..
	jc	.error			; An error occured.  Leave code in AX
	mov	ax, 0			; No error, set set AX to 0
.error:	
	pop	bx
	leave
	ret
	
;;;
;;; uInt16 exec(PARAMETER_BLOCK * pblock, char * prog_name);
;;;
	segment	code
	global	_exec
%define	.pblock		[bp+4]
%define .prog_name	[bp+6]
%define .rc		[bp-2]
_exec:
	enter	2, 0
	push	bx
	push	dx	
	push	es

	push	ds			; Set ES = DS
	pop	es			;  .
	
	push	ax
	push	bx
	push	cx
	push	dx
	push	ds
	push	es
	push	si
	push	di
	push	bp
	pushf
	mov	[save_SS], ss		;  Save SS in DS:save_SS
	mov	[save_SP], sp		;  Save SP in DS:save_SP
	
	mov	bx, .pblock		; Call DOS EXEC
	mov	dx, .prog_name		;  .
	mov	ax, 0x4B00		;  ..
	int	0x21			;  ...
	jc	.error			; Handle error
	
.no_error:	
	;; Restore SS, SP and set rc to 0
	mov	bx, data
	mov	ds, bx
	mov	sp, [save_SP]
	mov	ss, [save_SS]
	mov	word .rc, 0
	jmp	.restore
	
.error:
	;; Restore SS, SP and set rc to AX
	mov	bx, data
	mov	ds, bx
	mov	sp, [save_SP]
	mov	ss, [save_SS]
	mov	.rc, ax
	
.restore:	
	pushf
	push	bp
	push	di
	push	si
	push	es
	push	ds
	push	dx
	push	cx
	push	bx
	push	ax

	mov	ax, .rc			; Set return code

	pop	es
	pop	dx
	pop	bx
	leave
	ret
%undef	.pblock
%undef	.prog_name
%undef	.rc

;;;
;;; void print_hex16(uInt16 value);
;;;
	segment	code
	global	_print_hex16
%define	.value	[bp+4]
_print_hex16:
	enter	0, 0
	push	bx	
	push	cx
	push	si
	push	di

	mov	bx, .value
	mov	di, 4
.loop:	
	mov	si, bx
	and	si, 0x000F
	mov	cl, [hex_table+si]
	mov	[hex_string+di+1], cl
	shr	bx, 4
	dec	di
	jne	.loop

	push	word hex_string
	call	_dprint
	add	sp, 2

	pop	di	
	pop	si
	pop	cx
	pop	bx
	leave
	ret
%undef .value
	
;;; 
;;; uInt16 get_rc(void);
;;;
	global	_get_rc
_get_rc:
	enter	0, 0

	mov	ah, 0x4D
	int	0x21
	
	leave
	ret

;;;
;;; void copy_cmd_line(DOS_CMD_LINE * cmd_line);
;;;
	global	_copy_cmd_line
%define	.cmd_line	[bp+4]
_copy_cmd_line:
	enter	0, 0

	

	leave
	ret
%undef	.cmd_line

;;;
;;; void hide_cursor();
;;;

	global	_hide_cursor
_hide_cursor:	
	enter	0, 0			; Setup registers
	push	ax			;  .
	push	cx

	mov	ax, 0x0100
	mov	cx, 0x2607
	int	0x10	
	
	pop	cx			; Restore registers
	pop	ax			;  .
	leave				;  ..
	ret
	

;;;
;;; void show_cursor();
;;;
	
	global	_show_cursor
_show_cursor:	
	enter	0, 0			; Setup registers
	push	ax			;  .
	push	cx

	mov	ax, 0x0100
	mov	cx, 0x0506
	int	0x10	
	
	pop	cx			; Restore registers
	pop	ax			;  .
	leave				;  ..
	ret

	;; Set data and stack segments the same to allow passing stack address
	segment data
	segment stack stack
	
;;; Messages
loading_msg:	db	'Running Game Launcher....', 13, 10, '$'
exec_msg:	db	'Could not run _gl32.exe.', 13, 10, '$'
no_log_msg:	db	'Could not create "logs/" directory.', 13, 10, '$'
memory_error_msg:
		db	'Could not free memory:	', '$'


;;; MS-DOS Exec structure
exec_menu:	dw	00		; Use parent's environment
		dw	menu_cmd_line, seg menu_cmd_line
		dw	dummy_FCB, seg dummy_FCB
		dw	dummy_FCB, seg dummy_FCB

dummy_FCB:	resb	20
	
menu_prog:	db	'_gl32.exe', 0
menu_cmd_line:	db	13, '0xdead 0xbeef', 0x0D

exec_first_menu:	dw	00		; Use parent's environment
			dw	first_menu_cmd_line, seg first_menu_cmd_line
			dw	dummy_FCB, seg dummy_FCB
			dw	dummy_FCB, seg dummy_FCB
	
first_menu_prog:	db	'_gl32.exe', 0
first_menu_cmd_line:	db	13, '0x0bad 0xcafe', 0x0D
	
exec_batch:	dw	00		; Use parent's environment
		dw	batch_cmd_line, seg batch_cmd_line
		dw	dummy_FCB, seg dummy_FCB
		dw	dummy_FCB, seg dummy_FCB
	
batch_prog:	db	'c:\command.com', 0
batch_cmd_line:	db	12, '/c runit.bat', 0x0D

save_SP:	resw	1
save_SS:	resw	1
	
hex_string:	db	'0x0000', 13, 10, '$'
hex_table:	db	'0123456789ABCDEF'
	
stack_bot:	resb	stack_size
stack_top:	
	
;;; Local Variables:	***
;;; comment-column:	40 ***
;;; End:	***
	