#include "outline.h"
#include "gui/menu.h"
#include "gui/uimgr.h"
#include "gui/list.h"
#include "gui/separate.h"

using namespace Outline;
using namespace GUI;

/************************************************************************
 *
 * Menu
 *
 ************************************************************************/

Menu::Menu(int x, int y, int width, int height)
{
    p_x = x;
    p_y = y;
    p_width = width;
    p_height = height;

    p_title = "";

    p_subtitle = "";
    p_has_changed = true;

    p_ui = UI_Manager::get_menu_ui();
    p_ui->install_menu(this);
    p_title_list = p_ui->get_title_list();
    p_menu_list = p_ui->get_menu_list();
}

Menu::~Menu()
{
    delete p_ui;
}

int
Menu::get_x() const
{
    return p_x;
}

int
Menu::get_y() const
{
    return p_y;
}

int
Menu::get_width() const
{
    return p_width;
}

int
Menu::get_height() const
{
    return p_height;
}

void
Menu::set_title(const string & title)
{
    p_title = title;
    p_has_changed = true;
}

string
Menu::get_title() const
{
    return p_title;
}

void
Menu::set_subtitle(const string & subtitle)
{
    p_subtitle = subtitle;
    p_has_changed = true;
}

string
Menu::get_subtitle() const
{
    return p_subtitle;
}

void
Menu::set_list_model(const List_Model * model)
{
    p_menu_list->set_model(model);
}

void
Menu::set_selection_position(unsigned first_shown_index,
                             unsigned selected_index)
{
    p_menu_list->set_selection_position(first_shown_index,
                                        selected_index);
}

void
Menu::set_justification(Justification justification)
{
    p_justification = justification;
}

Justification
Menu::get_justification() const
{
    return p_justification;
}

unsigned
Menu::get_first_shown_index() const
{
    return p_menu_list->get_first_shown_index();
}

unsigned
Menu::get_selected_index() const
{
    return p_menu_list->get_selected_index();
}

void
Menu::set_selected_colors(int text, int outline)
{
    p_menu_list->set_selected_colors(text, outline);
}

void
Menu::set_unselected_colors(int text, int outline)
{
    // The title uses the same colors for selected and unselected so
    // no selection shows up.
    p_title_list->set_unselected_colors(text, outline);
    p_title_list->set_selected_colors(text, outline);

    p_menu_list->set_unselected_colors(text, outline);
}

void
Menu::line_up(Scroll_Options scroll_options)
{
    p_menu_list->line_up(scroll_options);
}

void
Menu::line_down(Scroll_Options scroll_options)
{
    p_menu_list->line_down(scroll_options);
}

void
Menu::page_up(Scroll_Options scroll_options)
{
    p_menu_list->page_up(scroll_options);
}

void
Menu::page_down(Scroll_Options scroll_options)
{
    p_menu_list->page_down(scroll_options);
}

void
Menu::hide_split()
{
    p_title_list->hide_top(1000);
    p_menu_list->hide_split();
}

void
Menu::hide_left()
{
    p_title_list->hide_top(1000);
    p_menu_list->hide_left();
}

void
Menu::hide_right()
{
    p_title_list->hide_top(1000);
    p_menu_list->hide_right();
}

void
Menu::hide_split_center_selected(int milliseconds)
{
    p_title_list->hide_top(1000);
    p_menu_list->hide_split_center_selected(milliseconds);
}

void
Menu::unhide()
{
    p_title_list->unhide();
    p_menu_list->unhide();
}

bool
Menu::update()
{
    if (p_has_changed)
    {
        p_ui->menu_changed();
        p_has_changed = false;
    }
    
    return p_ui->update();
}

void
Menu::draw(BITMAP * frame)
{
    p_ui->draw(frame);
}


/************************************************************************
 *
 * Menu_UI
 *
 ************************************************************************/

Menu_UI::~Menu_UI()
{
}


/************************************************************************
 *
 * Menu_Title_List_Adapter
 *
 ************************************************************************/

Menu_Title_List_Adapter::Menu_Title_List_Adapter(Menu * menu)
{
    p_menu = menu;
}

string
Menu_Title_List_Adapter::get_item_text_at(unsigned index) const
{
    if (index == 0)
    {
        return p_menu->get_title();
    }
    else // if (index == 1)
    {
        return p_menu->get_subtitle();
    }
}

Icon *
Menu_Title_List_Adapter::get_item_icon_at(unsigned index) const
{
    return 0;
}

unsigned
Menu_Title_List_Adapter::get_size() const
{
    return 2;
}

/************************************************************************
 *
 * Abstract_Menu_UI
 *
 ************************************************************************/

Abstract_Menu_UI::Abstract_Menu_UI()
{
    p_item_height = outline_text_height(UI_Manager::get_font());
}

Abstract_Menu_UI::~Abstract_Menu_UI()
{
    delete p_title_list;
    delete p_separator;
    delete p_menu_list;
}

void
Abstract_Menu_UI::menu_changed()
{
    Justification  justification;

    justification = p_menu->get_justification();
    p_title_list->set_justification(justification);
    p_separator->set_justification(justification);
    p_menu_list->set_justification(justification);
}

List *
Abstract_Menu_UI::get_title_list() const
{
    return p_title_list;
}

List *
Abstract_Menu_UI::get_menu_list() const
{
    return p_menu_list;
}

bool
Abstract_Menu_UI::update()
{
    bool updated;

    updated = false;

    updated |= p_title_list->update();
    updated |= p_separator->update();
    updated |= p_menu_list->update();

    return updated;
}

void
Abstract_Menu_UI::draw(BITMAP * frame)
{
    p_title_list->draw(frame);
    p_separator->draw(frame);
    p_menu_list->draw(frame);
}
