#include "gui/rot90.h"
#include "outline.h"

using namespace GUI;
using namespace Outline;


/************************************************************************
 *
 * Rotate_90_UI_Factory
 *
 ************************************************************************/

Label_UI *
Rotate_90_UI_Factory::create_label_ui()
{
    return new Rotate_90_Label_UI();
}

List_UI *
Rotate_90_UI_Factory::create_list_ui()
{
    return new Rotate_90_List_UI();
}

Menu_UI *
Rotate_90_UI_Factory::create_menu_ui()
{
    return new Rotate_90_Menu_UI();
}

Separator_UI *
Rotate_90_UI_Factory::create_separator_ui()
{
    return new Rotate_90_Separator_UI();
}


/************************************************************************
 *
 * Rotate_90_Label_UI
 *
 ************************************************************************/

Rotate_90_Label_UI::Rotate_90_Label_UI()
    : Abstract_Label_UI()
{
    p_90_degrees = itofix(64);
}

void
Rotate_90_Label_UI::install_ui(Label * label)
{
    int width;
    int height;

    destroy_bitmaps();

    p_label = label;
    width = outline_text_height(p_label->get_font());
    height = p_label->get_maximum_height();
    p_main_bitmap = ::create_bitmap(width, height);
    label_updated();
}

void
Rotate_90_Label_UI::label_updated()
{
    BITMAP * tmp;
    
    tmp = draw_horizontal(TEMPORARY_BITMAP, p_main_bitmap->h,
                          p_main_bitmap->w);
    p_label_bitmap = create_sub_bitmap(p_main_bitmap, 0, 0, tmp->h, tmp->w);
    p_label->set_size(p_label_bitmap->w, p_label_bitmap->h);

    clear_to_color(p_label_bitmap, p_label->get_background_color());
    // Move to upper right and rotate clockwise 90 degrees
    pivot_sprite(p_label_bitmap, tmp, p_label_bitmap->w, 0, 0, 0,
                 p_90_degrees);
    destroy_bitmap(tmp);
}


/************************************************************************
 *
 * Rotate_90_List_UI
 *
 ************************************************************************/

void
Rotate_90_List_UI::install_list(List * list)
{
    unsigned i;
    int new_width;

    p_list = list;
    p_x = p_list->get_x();
    p_y = p_list->get_y();
    p_width = p_list->get_width();
    p_height = p_list->get_height();

    p_num_visible_items = p_width / p_item_height;

    // Adjust the width to be a multiple of p_item_height and adjust
    // the x coordant appropriately.  This is necessary so that it
    // clips exactly on an item boundary.
    new_width = p_num_visible_items * p_item_height;
    p_x = p_x + (p_width - new_width);
    p_width = new_width;

    // Create the visible text items
    for (i = 0; i < p_num_visible_items; i++)
    {
        Label * label;

        label = new Label(p_item_height, p_height);
        p_visible_items.push_back(label);
    }

    // Create the off screen text items
    for (i = 0; i < p_num_visible_items; i++)
    {
        Label * label;

        label = new Label(p_item_height, p_height);
        p_offscreen_items.push_back(label);

    }

    // Add items to the container
    Label_List::iterator      j;

    for (j = p_visible_items.begin(); j != p_visible_items.end(); j++)
    {
        p_container->add_drawable((*j), DRAW_PRIORITY);
        p_container->add_updatable((*j), UPDATE_PRIORITY);
    }

    for (j = p_offscreen_items.begin(); j != p_offscreen_items.end(); j++)
    {
        p_container->add_drawable((*j), DRAW_PRIORITY);
        p_container->add_updatable((*j), UPDATE_PRIORITY);
    }

    hide_split(0);
}

int
Rotate_90_List_UI::get_justification_offset(Label * label)
{
    int offset;
    
    switch (p_justification)
    {
        case CENTER_JUSTIFIED:
            offset = (p_height / 2) - (label->get_height() / 2);
            break;

        case RIGHT_JUSTIFIED:
            offset = p_height - label->get_height();
            break;

        case LEFT_JUSTIFIED:
        default:
            offset = 0;
    }

    return offset;
}

inline int
Rotate_90_List_UI::get_indexed_x(int index)
{
    return (p_x + p_width) - (p_item_height * (index + 1));
}

void
Rotate_90_List_UI::hide_item_left(Label * item, int index, int milliseconds)
{
    int x;
    int y;

    x = get_indexed_x(index);
    y = p_y - p_height + get_justification_offset(item);

    item->set_location(x, y, milliseconds);
}

void
Rotate_90_List_UI::hide_item_right(Label * item, int index, int milliseconds)
{
    int x;
    int y;

    x = get_indexed_x(index);
    y = p_y + p_height + get_justification_offset(item);

    item->set_location(x, y, milliseconds);
}

void
Rotate_90_List_UI::hide_item_top(Label * item, int index, int milliseconds)
{
    int x;
    int y;

    x = get_indexed_x(index) + p_width;
    y = p_y + get_justification_offset(item);

    item->set_location(x, y, milliseconds);
}

void
Rotate_90_List_UI::center_item(Label * item, int milliseconds)
{
    int x;
    int y;

    x = (p_x + p_width / 2) - (p_item_height /2);
    y = p_y + get_justification_offset(item);

    item->set_location(x, y, milliseconds);
}

void
Rotate_90_List_UI::unhide_item(Label * item, int index, int milliseconds)
{
    int x;
    int y;

    x = get_indexed_x(index);
    y = p_y + get_justification_offset(item);

    item->set_location(x, y, milliseconds);
}

void
Rotate_90_List_UI::hide_offscreen_item(Label * item)
{
    int x;
    int y;

    // Offscreen items are "parked" just above the list
    x = p_x +p_width + p_item_height;
    y = p_y;

    item->set_location(x, y, 0);
}

/************************************************************************
 *
 * Rotate_90_Menu_UI
 *
 ************************************************************************/

void
Rotate_90_Menu_UI::install_menu(Menu * menu)
{
    int x;
    int y;
    int width;
    int height;
    int title_width;
    int x_offset;
        
    p_menu = menu;
    x = p_menu->get_x();
    y = p_menu->get_y();
    width = p_menu->get_width();
    height = p_menu->get_height();
    x_offset = width;

    title_width = 2 * p_item_height;
    x_offset -= title_width;
    p_title_list = new List(x + x_offset, y, title_width, height);
    p_title_model = new Menu_Title_List_Adapter(p_menu);
    p_title_list->set_model(p_title_model);
    p_title_list->hide_top(0);

    x_offset -= SEPARATOR_HEIGHT;
    p_separator = new Separator(x + x_offset, y, SEPARATOR_HEIGHT, height);

    p_menu_list = new List(x, y, x_offset, height);
}


/************************************************************************
 *
 * Rotate_90_Separator_UI
 *
 ************************************************************************/

int
Rotate_90_Separator_UI::get_inner_x()
{
    return (p_bounding_box->get_x() + p_bounding_box->get_width() / 4);
}

int
Rotate_90_Separator_UI::get_inner_y()
{
    int y_offset;

    switch (p_separator->get_justification())
    {
        case LEFT_JUSTIFIED:
            y_offset = 0;
            break;

        case RIGHT_JUSTIFIED:
            y_offset = p_bounding_box->get_height() / 2;
            break;

        case CENTER_JUSTIFIED:
        default:
            y_offset = p_bounding_box->get_height() / 4;
            break;
    }

    return (p_bounding_box->get_y() + y_offset);
}
