#ifndef _FILEUTIL_H_
#define _FILEUTIL_H_

#include <string>
#include <vector>
using namespace std;

string  get_extension(const string & path);
string  remove_extension(const string & path);
void    split_extension(const string & path, string & base_name,
                        string & extension);
void    add_backslash(string & path);
void    fix_filename_slashes(string & path);
bool    is_absolute_path(const string & path);

void    quote_long_file_name(string & file_name);

void    fix_file_name_case(string & file_name);

class Config_Section;

class Config_File
{
  public:
    Config_File(const string & file_name);

    Config_Section *    get_section(const string & section);

  private:
    string      p_file_name;
};

class Config_Section
{
  public:
    virtual ~Config_Section() = 0;
    virtual string      get_string(const string & field,
                                   const string & default_value) = 0;
    virtual void        set_string(const string & field,
                                   const string & value) = 0;
    /**
     * Returns the field as a file name.  A file name is exactly like
     * a string, except the case may be altered on some platforms.
     */
    virtual string      get_file_name(const string & field,
                                      const string & default_value) = 0;

    virtual bool        get_bool(const string & field, bool default_value) = 0;
    virtual void        set_bool(const string & field, bool value) = 0;

    virtual int         get_int(const string & field, int default_value) = 0;
    virtual void        set_int(const string & field, int value) = 0;

    virtual vector<string> *    get_string_list(const string & field) = 0;

    virtual vector<string> *    get_file_name_list(const string & field) = 0;
};

class Config_Section_Impl : public Config_Section
{
  public:
    Config_Section_Impl(const string & section);
    virtual ~Config_Section_Impl();

    string      get_string(const string & field,
                           const string & default_value);
    void        set_string(const string & field,
                           const string & value);

    string      get_file_name(const string & file,
                              const string & default_value);
    
    bool        get_bool(const string & field, bool default_value);
    void        set_bool(const string & field, bool value);
    
    int         get_int(const string & field, int default_value);
    void        set_int(const string & field, int value);

    vector<string> *    get_string_list(const string & field);
    vector<string> *    get_file_name_list(const string & field);

  private:
    string      p_section;
};

void    set_config_file(const string & config_file);
string  get_config_string(const string & section, const string & name,
                          const string & default_value);
void    set_config_string(const string & section, const string & name,
                          const string & value);

bool    get_config_bool(const string & section, const string & name,
                        bool default_value);

void    set_config_bool(const string & section, const string & name,
                        bool value);

int     get_config_int(const string & section, const string & field,
                       int default_value);
void    set_config_int(const string & section, const string & field,
                       int value);

char ** get_config_argv(const string & section, const string & field,
                        int * argc);

bool    file_exists(const string & file);

bool    directory_exists(const string & exists);

bool    create_directory(const string & exists);

long    file_size(const string & file);

#endif
