#ifndef _GUI_ICON_H_
#define _GUI_ICON_H_

#include "allegro.h"

namespace GUI
{
    /**
     * An interface for fixed size graphics.
     */
    class Icon
    {
      public:
        /**
         * Virtual destructor for interface.
         */
        virtual ~Icon();

        /**
         * Gets the width of the icon.
         *
         * @return This icons width, in pixels.
         */
        virtual int get_icon_width() const = 0;

        /**
         * Gets the height of the icon.
         *
         * @return This icons height, in pixels.
         */
        virtual int get_icon_height() const = 0;

        /**
         * Draws the icon to a location on a bitmap.
         *
         * @param bitmap Bitmap to draw on.
         * @param x The x coordinate of the bitmap.
         * @param y The y coordinate of the bitmap.
         */
        virtual void paint_icon(BITMAP * bitmap, int x, int y) const = 0;
    };

    /**
     * An icon implemented using a bitmap.
     */
    class Bitmap_Icon : public Icon
    {
      public:
        /**
         * Constuct a new icon with the given bitmap.  Ownership of
         * the bitmap will belong to the icon, so the user should not
         * destroy the bitmap.
         */
        Bitmap_Icon(BITMAP * icon);

        /**
         * Destroys the bitmap associated with this icon.
         */
        virtual ~Bitmap_Icon();

        virtual int get_icon_width() const;

        virtual int get_icon_height() const;

        virtual void paint_icon(BITMAP * bitmap, int x, int y) const;

      private:
        /** The bitmap to be used as an icon. */
        BITMAP * p_icon;
    };
};

#endif

/* Local Variables: */
/* mode: c++ */
/* End: */
