#ifndef _GUI_SEPARATE_H_
#define _GUI_SEPARATE_H_

#include "gui/comp.h"
#include "gui/util.h"

namespace GUI
{
    class Separator_UI;

    /**
     * Displayed as a rectangle that takes up half the bounding box.
     * It is always centered from top to bottom.  The justification
     * detemerines the left to right location.
     */
    class Separator : public Component
    {
      public:
        /**
         * Constructs a separator from a bounding box.
         *
         * @param x X-coordinate of upper left corner.
         * @param y Y-coordinate of upper left corner.
         * @param width Width.
         * @param height Height.
         */
        Separator(int x, int y, int width, int height);

        /**
         * Constructs a separator from a bounding box.
         *
         * @param bounding_box Bounding box.
         */
        Separator(const Rectangle & bounding_box);

        /**
         * Destructor.
         */
        ~Separator();
        
        /**
         * Gets the bounding box.
         *
         * @return Bounding box.
         */
        const Rectangle * get_bounding_box() const;

        void set_justification(Justification justification);

        Justification get_justification() const;

        virtual bool update();

        virtual void draw(BITMAP * frame);

      private:
        Separator_UI * p_ui;
        
        bool p_changed;

        Rectangle p_bounding_box;

        Justification p_justification;

    };

    class Separator_UI : public Drawable
    {
      public:
        virtual ~Separator_UI();

        virtual void install_separator(Separator * separator) = 0;

        virtual void separator_changed() = 0;

        virtual void draw(BITMAP * frame) = 0;
    };

    class Abstract_Separator_UI : public Separator_UI
    {
      public:
        Abstract_Separator_UI();
            
        virtual void install_separator(Separator * separator);

        virtual void separator_changed();

        virtual void draw(BITMAP * frame);

      protected:
        static const int OUTLINE_SIZE = 2;

        virtual int get_inner_x() = 0;

        virtual int get_inner_y() = 0;

        Separator * p_separator;
        
        const Rectangle * p_bounding_box;
        
        Rectangle p_outer_box;
        
        Rectangle p_inner_box;

        int p_black;

        int p_white;
    };
};

#endif

/* Local Variables: */
/* mode: c++ */
/* End: */
