#ifndef _LOGFILE_H_
#define _LOGFILE_H_

#include <string>
#include <fstream>
using namespace std;

#define LOG_FILE_ENABLE_DEBUG   1

namespace LogFile
{
    using       std::ofstream;

    class Log_File
    {
      public:
        Log_File(const string & file_name);
        
        void    error(const string & error_string);
        void    warning(const string & warning_string);
        void    info(const string & info_string);
        void    debug(const string & debug_string);

      private:
        static const int    NUMBER_OF_LOG_BACKUPS = 5;
        void    write_line(const string & type, const string & line);
        ofstream        p_log_stream;
    };

    class Log_Rotate_Event
    {
      public:
        Log_Rotate_Event(int completed_rotations, int total_rotations);
        int get_completed_rotations() const;
        int get_total_rotations() const;

      private:
        int         p_completed_rotations;
        int         p_total_rotations;
    };

    class Log_Rotate_Listener
    {
      public:
        virtual ~Log_Rotate_Listener();
        virtual void rotate_progress(const Log_Rotate_Event * event) = 0;
    };


    extern Log_File *   gl_log;
    void    open_log_file(const string & file_name);
    void    rotate_log_files(string base_name, int number_of_backups,
                             Log_Rotate_Listener * listener);
    void    replace_extension(char * dest, const char * src,
                              const char * ext, int size);

#if (! LOG_FILE_ENABLE_DEBUG)
    inline Log_File::debug(const string & debug_string) { }
#endif
    
};


#endif

/* Local Variables: */
/* mode: c++ */
/* End: */
