#if USE_TTF
#include "allegttf.h"
#endif
#include "view.h"
#include "logfile.h"
#include "outline.h"

using namespace GameLauncher;

using Outline::outline_text_height;
using Outline::outline_text_length;
using Outline::outline_textout;

using LogFile::gl_log;


RGB_MAP Text_VC::s_rgb_table;
bool    Text_VC::s_initialized_palette  = false;

Text_VC::Text_VC(Engine * engine, int width, int height, FONT * font)
    : Movable_Bitmap(engine, width, height)
{
    p_font      = font;
    p_icon      = 0;
    p_body_color        = makecol(255, 255, 255);
    p_outline_color     = makecol(0, 0, 0);
    strcpy(p_text, "");

    if (! s_initialized_palette)
    {
        PALETTE palette;
        RGB     black = { 0, 0, 0 };

        generate_332_palette(palette);
        palette[0] = black;

        set_palette(palette);

        create_rgb_table(&s_rgb_table, palette, NULL);
        rgb_map = &s_rgb_table;
#if (USE_TTF == 1)
        antialias_init(palette);
#endif
        s_initialized_palette = true;
    }
}

void
Text_VC::set_colors(int body_color, int outline_color)
{
    p_body_color        = body_color;
    p_outline_color     = outline_color;
    update_bitmap();
}

void
Text_VC::set_contents(const string & text, BITMAP * icon = 0)
{
    char *      end;
    int         max_width;
    int         pixel_width;

    p_icon      = icon;
    strncpy(p_text, text.c_str(), MAX_TEXT_LEN);
    pixel_width = text_length(p_font, p_text) + OUTLINE_PADDING;
    
    max_width = p_text_width;
    // Save space for screen shot icon
    if (p_icon != 0)
    {
        max_width -= p_icon->w;
    }

    // Chop off a character at a time to make sure it fits
    end = &p_text[strlen(p_text) - 1];
    while (pixel_width > max_width)
    {
        *end = '\0';
        end--;
        pixel_width = text_length(p_font, p_text) + OUTLINE_PADDING;
    }

    update_bitmap();

    return;
}


BITMAP *
Text_VC::get_outline_bmp()
{
    BITMAP * text_bmp;
    int t_width, t_height;

    t_height    = outline_text_height(p_font);
    t_width     = outline_text_length(p_font, p_text);
    
    text_bmp    = create_bitmap(t_width, t_height);
    clear_to_color(text_bmp, makecol(255, 0, 255));
    outline_textout(text_bmp, p_font, p_text, p_body_color, p_outline_color);
    return text_bmp;
}

#if 0
BITMAP *
Text_VC::get_outline_bmp()
{
    int offsets[][2] = 
    {
        { 2,  1},
        { 2,  0},
        { 2, -1},
        {-2,  1},
        {-2,  0},
        {-2, -1},
        { 1,  2},
        { 0,  2},
        {-1,  2},
        { 1, -2},
        { 0, -2},
        {-1, -2},
        { 0,  0}
    };
    int i, t_height, t_width, trans_col;
    BITMAP *    text_bmp;
    BITMAP *    tmp_bmp;

    t_height    = text_height(p_font) + OUTLINE_PADDING;
    t_width     = text_length(p_font, p_text) + OUTLINE_PADDING*2;
    text_mode(-1);

#if 0
    trans_col   = makecol(255, 0, 255);
    text_bmp    = create_bitmap(t_width, t_height);
    tmp_bmp     = create_bitmap(t_width, t_height);
#else
    int outline_color;
    int body_color;
    RGB color;

    color.r = getr(p_outline_color) >> 2;
    color.g = getg(p_outline_color) >> 2;
    color.b = getb(p_outline_color) >> 2;
    set_color(254, &color);
    outline_color = makecol8(getr(p_outline_color),
                             getg(p_outline_color),
                             getb(p_outline_color));
    outline_color = 254;
    body_color    = makecol8(getr(p_body_color),
                             getg(p_body_color),
                             getb(p_body_color));

#if 0
    color.r = getr(p_outline_color) >> 2;
    color.g = getg(p_outline_color) >> 2;
    color.b = getb(p_outline_color) >> 2;
    set_color(0, &color);
    outline_color = 0;

    color.r = 0;
    color.g = 0;
    color.b = 0;
    set_color(0, &color);

    color.r = getr(p_body_color) >> 2;
    color.g = getg(p_body_color) >> 2;
    color.b = getb(p_body_color) >> 2;
    set_color(255, &color);
#endif

    trans_col   = 0;

    text_bmp    = create_bitmap(t_width, t_height);
    tmp_bmp     = create_bitmap_ex(8, t_width, t_height);

#endif

    clear_to_color(text_bmp, makecol(255, 0, 255));
    clear_to_color(tmp_bmp, trans_col);
#if 0
    textout_centre(tmp_bmp, p_font, p_text, t_width/2, OUTLINE_PADDING/2,
                   p_outline_color);
#else
    textout_centre(tmp_bmp, p_font, p_text, t_width/2, OUTLINE_PADDING/2,
                   outline_color);
#endif

    i = 0;
    while (! ((offsets[i][0] == 0) && (offsets[i][1] == 0)))
    {
        draw_sprite(text_bmp, tmp_bmp, offsets[i][0], offsets[i][1]);
        i++;
    }
    draw_sprite(text_bmp, tmp_bmp, offsets[i][0], offsets[i][1]);

    clear_to_color(tmp_bmp, trans_col);
#if 0
    textout_centre(tmp_bmp, p_font, p_text, t_width/2, OUTLINE_PADDING/2,
                   p_body_color);
    draw_sprite(text_bmp, tmp_bmp, 0, 0);
#else
    textout_centre(text_bmp, p_font, p_text, t_width/2, OUTLINE_PADDING/2,
                   p_body_color);
#endif
    destroy_bitmap(tmp_bmp);
    
    return text_bmp;
}
#endif

HText_VC::HText_VC(Engine * engine, int width, int height, FONT * font)
    : Text_VC(engine, width, height, font)
{
    p_text_width = width;
}

void
HText_VC::update_bitmap()
{
    BITMAP *    outline_bmp;
    int         left_x;

    clear_to_color(p_bitmap, makecol(255, 0, 255));
    outline_bmp = get_outline_bmp();
    if (p_icon == 0)
        left_x = p_bitmap->w/2 - outline_bmp->w/2;
    else
        left_x = p_bitmap->w/2 - (outline_bmp->w + p_icon->w)/2;
    draw_sprite(p_bitmap, outline_bmp, left_x, 0);
    if (p_icon != 0)
    {
        draw_sprite(p_bitmap, p_icon, left_x + outline_bmp->w,
                    (p_bitmap->h/2 - p_icon->h/2));
    }
    return;
}

VText_VC::VText_VC(Engine * engine, int width, int height, FONT * font)
    : Text_VC(engine, width, height, font)
{
    p_text_width = height;
}

void
VText_VC::update_bitmap()
{
    BITMAP *    outline_bmp;
    BITMAP *    tmp;
    int         bottom_y;

    clear_to_color(p_bitmap, makecol(255, 0, 255));
    outline_bmp = get_outline_bmp();
    if (p_icon == 0)
        bottom_y = p_bitmap->h/2 + outline_bmp->w/2;
    else
        bottom_y = p_bitmap->h/2 + (outline_bmp->w + p_icon->w)/2;

    tmp = create_bitmap(outline_bmp->w, outline_bmp->w);
    clear_to_color(tmp, makecol(255, 0, 255));
#if 0
    rotate_sprite(tmp, outline_bmp, -outline_bmp->w/2 + outline_bmp->h/2,
                  outline_bmp->w/2 - outline_bmp->h/2, itofix(192));
#else
    pivot_sprite(tmp, outline_bmp, 1, -1, outline_bmp->w, 0,
                 itofix(192));
#endif
    blit(tmp, p_bitmap, 0, 0, 0, bottom_y - outline_bmp->w, tmp->w, tmp->h);

    destroy_bitmap(tmp);
    if (p_icon != 0)
    {
        int i = ((p_icon->w > p_icon->h) ? p_icon->w : p_icon->h);
        tmp = create_bitmap(i, i);
        clear_to_color(tmp, makecol(255, 0, 255));
#if 0
        rotate_sprite(tmp, p_icon, -p_icon->w/2 + p_icon->h/2,
                      p_icon->w/2 - p_icon->h/2, itofix(192));
#else
        pivot_sprite(tmp, p_icon, 1, -1, p_icon->w, 0, itofix(192));
#endif
        blit(tmp, p_bitmap, 0, 0, (p_bitmap->w/2 - p_icon->h/2),
             bottom_y - outline_bmp->w - p_icon->w,
             tmp->w, tmp->h);
        destroy_bitmap(tmp);
    }
    
    destroy_bitmap(outline_bmp);

    return;
}
