#include "outline.h"

using namespace Outline;


/** Padding the outline will add. */
static const int OUTLINE_PADDING            = 4;

/*
 * Use this offset pattern:
 *
 *       XXX
 *      X   X
 *      X + X
 *      X   X
 *       XXX
 */
static int offsets[][2] = 
{
    { 2,  1},
    { 2,  0},
    { 2, -1},
    {-2,  1},
    {-2,  0},
    {-2, -1},
    { 1,  2},
    { 0,  2},
    {-1,  2},
    { 1, -2},
    { 0, -2},
    {-1, -2},
    { 0,  0}
};



int
Outline::outline_text_height(const FONT * font)
{
    return text_height(font) + OUTLINE_PADDING;
}

int
Outline::outline_text_length(const FONT * font, const string & text)
{
    return text_length(font, text.c_str()) + OUTLINE_PADDING * 2;
}


void
Outline::outline_textout(BITMAP * bitmap, const FONT * font,
                         const string & text, int foreground_color,
                         int outline_color)
{
    BITMAP *    tmp_bmp;
    int t_height;
    int t_width;
    int outline_color8;
    int trans_color;
    int i;
    RGB color;

    t_height    = outline_text_height(font);
    t_width     = outline_text_length(font, text);

    text_mode(-1);

    // Convert 32-bit color to 8-bit, and store it in the palette at
    // color 254.
    color.r = getr(outline_color) >> 2;
    color.g = getg(outline_color) >> 2;
    color.b = getb(outline_color) >> 2;
    set_color(254, &color);
    outline_color8  = 254;

    trans_color = 0;
    tmp_bmp = create_bitmap_ex(8, t_width, t_height);
    
    clear_to_color(tmp_bmp, trans_color);

    textout_centre(tmp_bmp, font, text.c_str(), t_width/2, OUTLINE_PADDING/2,
                   outline_color8);
    
    i = 0;
    while (! ((offsets[i][0] == 0) && (offsets[i][1] == 0)))
    {
        draw_sprite(bitmap, tmp_bmp, offsets[i][0], offsets[i][1]);
        i++;
    }
    draw_sprite(bitmap, tmp_bmp, offsets[i][0], offsets[i][1]);

    textout_centre(bitmap, font, text.c_str(), t_width/2, OUTLINE_PADDING/2,
                   foreground_color);
    
    destroy_bitmap(tmp_bmp);
}
