#include "view.h"

using namespace GameLauncher;

/************************************************************************
 *
 * Screen_Saver_VC
 *
 ************************************************************************/

Screen_Saver_VC::Screen_Saver_VC(Engine * engine, FONT * font)
{
    p_engine    = engine;
    p_listener  = 0;
    p_is_active = false;
    p_start_after   = 0;
    p_font      = font;
    p_text      = "";
    p_text_location_timer =
        p_engine->get_timer_factory()->create_timer();
}

Screen_Saver_VC::~Screen_Saver_VC()
{
}

void
Screen_Saver_VC::set_random_text_location()
{
    p_text_x = rand() % (SCREEN_W - text_length(p_font, p_text.c_str()));
    p_text_y = rand() % (SCREEN_H - text_height(p_font));
}

bool
Screen_Saver_VC::update()
{
    if (p_start_after == 0)
        return false;
    
    if (p_is_active)
    {
        // Check for activity.
        if (p_engine->get_idle_timer() < p_start_after)
        {
            p_text_location_timer->disable();
            p_is_active = false;
            fire_changed_state_event(p_is_active);
        }
        else
        {
            if (p_text_location_timer->expired())
            {
                set_random_text_location();
            }
        }
    }
    else
    {
        // Check for inactivity.
        if (p_engine->get_idle_timer() >= p_start_after)
        {
            p_is_active = true;
            fire_changed_state_event(p_is_active);
            set_random_text_location();
            p_text_location_timer->set_interval(1000*3);
        }
    }

    return false;
}

void
Screen_Saver_VC::draw(BITMAP * frame)
{
    int black;
    int white;
    
    black   = makecol(0, 0, 0);
    white   = makecol(255, 255, 255);
    
    clear(frame);
    text_mode(black);
    textout(frame, p_font, p_text.c_str(), p_text_x, p_text_y, white);
}

void
Screen_Saver_VC::set_screen_saver_listener(Screen_Saver_Listener * listener)
{
    p_listener = listener;
}

void
Screen_Saver_VC::set_start_after(unsigned int seconds)
{
    p_start_after = seconds;
}

void Screen_Saver_VC::set_text(string text)
{
    p_text = text;
}

void
Screen_Saver_VC::fire_changed_state_event(bool is_active)
{
    if (p_listener != 0)
    {
        Screen_Saver_Event  e(is_active);

        p_listener->screen_saver_changed_state(&e);
    }
}

/************************************************************************
 *
 * Screen_Saver_Event
 *
 ************************************************************************/

Screen_Saver_Event::Screen_Saver_Event(bool is_active)
{
    p_is_active = is_active;
}

bool
Screen_Saver_Event::is_active() const
{
    return p_is_active;
}

/************************************************************************
 *
 * Screen_Saver_Listener
 *
 ************************************************************************/

Screen_Saver_Listener::~Screen_Saver_Listener()
{
}
