#include <string>
#include "allegro.h"
#include "allegttf.h"

#define OUTLINE_PADDING 4

static FONT *   p_font;
BITMAP *        p_icon;
BITMAP *        p_bitmap;
int             p_body_color;
int             p_outline_color;
char *          p_text;

static RGB_MAP d_rgb_table;
static bool d_initialized = false;
static PALETTE d_palette;

void
update_bitmap();

BITMAP *
get_outline_bmp();

int
main(void)
{
    allegro_init();
    install_keyboard();

    set_color_depth(16);
    set_gfx_mode(GFX_AUTODETECT, 640, 480, 0, 0);

    p_font = load_ttf_font("font.ttf", 20, ALLEGTTF_REALSMOOTH);
    p_bitmap = create_bitmap(text_height(p_font) + OUTLINE_PADDING, 480);
    p_icon = 0;
    p_text = "Bobble Bobble";
    p_outline_color = makecol(192, 192, 64);
    p_body_color = makecol(0, 0, 0);

    RGB         black = { 0, 0, 0 };

    generate_332_palette(d_palette);
    d_palette[0] = black;

    set_palette(d_palette);

    create_rgb_table(&d_rgb_table, d_palette, NULL);
    rgb_map = &d_rgb_table;
    antialias_init(d_palette);

    textout_centre(screen, p_font, "Test", SCREEN_W/2, SCREEN_H/2,
                   makecol(255, 255, 255));
    update_bitmap();
    draw_sprite(screen, p_bitmap, 0, 0);
    
    while (!keypressed())
        ;
}


void
update_bitmap()
{
    BITMAP *    outline_bmp;
    BITMAP *    tmp;
    int         bottom_y;

    clear_to_color(p_bitmap, makecol(255, 0, 255));
    outline_bmp = get_outline_bmp();
    if (p_icon == 0)
        bottom_y = p_bitmap->h/2 + outline_bmp->w/2;
    else
        bottom_y = p_bitmap->h/2 + (outline_bmp->w + p_icon->w)/2;

    // tmp = create_bitmap(outline_bmp->w, outline_bmp->w);
    tmp = create_bitmap(SCREEN_W, SCREEN_H);

    clear_to_color(tmp, makecol(255, 0, 255));
#if 0
    rotate_sprite(tmp, outline_bmp, -outline_bmp->w/2 + outline_bmp->h/2,
                  outline_bmp->w/2 - outline_bmp->h/2, itofix(192));
    blit(tmp, p_bitmap, 0, 0, 0, bottom_y - outline_bmp->w, tmp->w, tmp->h);
#else
    pivot_sprite(tmp, outline_bmp, 1, -1, outline_bmp->w, 0,
                 itofix(192));
    blit(tmp, p_bitmap, 0, 0, 0, bottom_y - outline_bmp->w, tmp->w, tmp->h);
#endif

    draw_sprite(screen, tmp, 0, 0);
    while (!keypressed())
        ;
    clear_keybuf();

    destroy_bitmap(tmp);
    if (p_icon != 0)
    {
        int i = ((p_icon->w > p_icon->h) ? p_icon->w : p_icon->h);
        tmp = create_bitmap(i, i);
        clear_to_color(tmp, makecol(255, 0, 255));
        rotate_sprite(tmp, p_icon, -p_icon->w/2 + p_icon->h/2,
                      p_icon->w/2 - p_icon->h/2, itofix(192));
        blit(tmp, p_bitmap, 0, 0, p_icon->w/2,
             bottom_y - outline_bmp->w - p_icon->w,
             tmp->w, tmp->h);
        destroy_bitmap(tmp);
    }
    
    destroy_bitmap(outline_bmp);

    return;
}


#if 0
#define DEBUG(_X_) { draw_sprite(screen, _X_, debug_x, debug_y); debug_y += t_height; if (debug_y > SCREEN_H) { debug_y = 0; debug_x += t_width; } }
#else
#define DEBUG(_X_)
#endif

BITMAP *
get_outline_bmp()
{
    int offsets[][2] = 
    {
        { 2,  1},
        { 2,  0},
        { 2, -1},
        {-2,  1},
        {-2,  0},
        {-2, -1},
        { 1,  2},
        { 0,  2},
        {-1,  2},
        { 1, -2},
        { 0, -2},
        {-1, -2},
        { 0,  0}
    };
    int i, t_height, t_width, trans_col;
    BITMAP *    text_bmp;
    BITMAP *    tmp_bmp;
    static int debug_x = 0;
    static int debug_y = 0;

    t_height    = text_height(p_font) + OUTLINE_PADDING*2;
    t_width     = text_length(p_font, p_text) + OUTLINE_PADDING*2;
    text_mode(-1);

#if 0
    trans_col   = makecol(255, 0, 255);
    text_bmp    = create_bitmap(t_width, t_height);
    tmp_bmp     = create_bitmap(t_width, t_height);
#else
    int outline_color;
    int body_color;
    RGB color;

    color.r = getr(p_outline_color) >> 2;
    color.g = getg(p_outline_color) >> 2;
    color.b = getb(p_outline_color) >> 2;
    set_color(254, &color);
    outline_color = makecol8(getr(p_outline_color),
                             getg(p_outline_color),
                             getb(p_outline_color));
    outline_color = 254;
    body_color    = makecol8(getr(p_body_color),
                             getg(p_body_color),
                             getb(p_body_color));

#if 0
    color.r = getr(p_outline_color) >> 2;
    color.g = getg(p_outline_color) >> 2;
    color.b = getb(p_outline_color) >> 2;
    set_color(0, &color);
    outline_color = 0;

    color.r = 0;
    color.g = 0;
    color.b = 0;
    set_color(0, &color);

    color.r = getr(p_body_color) >> 2;
    color.g = getg(p_body_color) >> 2;
    color.b = getb(p_body_color) >> 2;
    set_color(255, &color);
#endif

    trans_col   = 0;

    text_bmp    = create_bitmap(t_width, t_height);
    tmp_bmp     = create_bitmap_ex(8, t_width, t_height);
#endif

    clear_to_color(text_bmp, makecol(255, 0, 255));
    clear_to_color(tmp_bmp, trans_col);
#if 0
    textout_centre(tmp_bmp, p_font, p_text, t_width/2, OUTLINE_PADDING/2,
                   p_outline_color);
#else
    textout_centre(tmp_bmp, p_font, p_text, t_width/2, OUTLINE_PADDING/2,
                   outline_color);
#endif

    i = 0;
    while (! ((offsets[i][0] == 0) && (offsets[i][1] == 0)))
    {
        draw_sprite(text_bmp, tmp_bmp, offsets[i][0], offsets[i][1]);
        i++;
        DEBUG(text_bmp);
    }
    draw_sprite(text_bmp, tmp_bmp, offsets[i][0], offsets[i][1]);
    DEBUG(text_bmp);

    clear_to_color(tmp_bmp, trans_col);
#if 0
    textout_centre(tmp_bmp, p_font, p_text, t_width/2, OUTLINE_PADDING/2,
                   p_body_color);
    draw_sprite(text_bmp, tmp_bmp, 0, 0);
#else
    textout_centre(text_bmp, p_font, p_text, t_width/2, OUTLINE_PADDING/2,
                   p_body_color);
#endif
    destroy_bitmap(tmp_bmp);
    
    return text_bmp;
}
